@@ -4,7 +4,7 @@
 # Pod stripped from pm file by OODoc 2.01.
 package Apache::Solr::XML;
 use vars '$VERSION';
-$VERSION = '0.93';
+$VERSION = '0.94';
 
 use base 'Apache::Solr';
 
@@ -152,9 +152,7 @@ sub request($$;$$)
     my $resp = $self->SUPER::request($url, $result, $body, $body_ct);
     my $ct   = $resp->content_type;
 #warn $resp->as_string;
-    $ct =~ m/xml/i
-        or error __x"answer from solr server is not xml but {type}"
-            , type => $ct;
+    $ct =~ m/xml/i or return $result;
 
     my $dec = $self->xmlsimple
        ->XMLin($resp->decoded_content || $resp->content);
